package org.heatdeath.freewiki;

import java.io.*;
import java.net.*;

public class FreeWikiURLConnection extends URLConnection {
    private static WikiManager wikiManager = null;
    private WikiNodeData.RevisionData revisionData = null;

    private static WikiManager getWikiManager() throws IOException {
	if (wikiManager == null) {
	    if (System.getProperty("freenet.serverAddress") == null) {
		throw new IOException("freenet.serverAddress system property is " +
				      "required for freewiki URLs.");
	    }
	    if (System.getProperty("freewiki.htl") == null) {
		throw new IOException("freewiki.htl system property is " +
				      "required for freewiki URLs.");
	    }
	    try {
		wikiManager = new WikiManager(
		        System.getProperty("freewiki.serverAddress"),
		        Integer.parseInt(System.getProperty("freewiki.htl")));
	    } catch (NumberFormatException ex) {
		throw new IOException("freewiki.htl System property must be " +
				      "numeric.");
	    }
	}
	return wikiManager;
    }

    private String wikiName() {
	return wikiName(url);
    }

    private String nodeName() {
	return nodeName(url);
    }

    public static String wikiName(URL url) {
	return url.getHost();
    }

    public static String nodeName(URL url) {
	return url.getPath().substring(1);
    }

    public FreeWikiURLConnection(URL url) {
	super(url);
    }

    public void connect() throws IOException {
	System.out.println("connect()");
	try {
	    WikiNodeData data = getWikiManager().getNode(wikiName(), nodeName());
	    if (url.getQuery() == null) {
		revisionData = data.getCurrentRevision();
	    } else {
		revisionData = (WikiNodeData.RevisionData) data.getRevisions().get(
		        Integer.parseInt(url.getQuery()));
	    }
	} catch (NumberFormatException ex) {
	    throw new IOException("Revision number must be numeric.");
	} catch (WikiStoreException ex) {
	    throw new IOException(ex.toString());
	}
    }

    public String getContentType() {
	return "text/html; charset=UTF-8";
    }

    public InputStream getInputStream() throws IOException {
	System.out.println("getInputStream()");
	if (revisionData == null) {
	    connect();
	}
	try {
	    return new ByteArrayInputStream(
		    revisionData.getContent().getBytes("UTF-8"));
	} catch (UnsupportedEncodingException ex) {
	    ex.printStackTrace();
	    throw new RuntimeException(ex.toString());
	}
    }
}
