package org.heatdeath.freewiki;

import java.util.*;
import java.text.*;

public class WikiNodeData {
    private String nodeName;
    private String wikiName;
    private List revisions;

    public static class RevisionData {
	private String contents;
	private int revisionNumber;
	private Date date;

	public RevisionData(int revisionNumber, Date date, String contents) {
	    this.contents = contents;
	    this.revisionNumber = revisionNumber;
	    this.date = date;
	}

	public String getContent() {
	    return contents;
	}

	public int getRevisionNumber() {
	    return revisionNumber;
	}

	public Date getDate() {
	    return date;
	}

	public String toString() {
	    if (date == null) {
		return Integer.toString(revisionNumber);
	    } else {
		return revisionNumber + ": " +
		    new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").format(date);
	    }
	}
    }

    public WikiNodeData(String wikiName, String nodeName, List revisions) {
	this.wikiName = wikiName;
	this.nodeName = nodeName;
	this.revisions = revisions;
    }

    public String getWikiName() {
	return wikiName;
    }

    public String getName() {
	return nodeName;
    }

    public List getRevisions() {
	return revisions;
    }

    public RevisionData getCurrentRevision() {
	if (revisions.size() == 0) {
	    return new RevisionData(0, null, "");
	}
	return (RevisionData) revisions.get(revisions.size() - 1);
    }

    public String getContent() {
	return getCurrentRevision().getContent();
    }

    public int getRevisionNumber() {
	return getCurrentRevision().getRevisionNumber();
    }
}
