package org.heatdeath.freewiki.gui;

import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.event.*;
import org.heatdeath.freewiki.*;

public class FreeWikiApplication {
    private CachedWikiManager wikiManager;
    private String currentWikiName;
    private String waitingForWikiName = null;
    private String waitingForNodeName = null;
    private WikiNodeData currentNodeData = null;
    private ArrayList history = new ArrayList();
    private int historyIndex = -1;
    private JFrame frame;
    private JDialog saveDialog;
    private JLabel saveLabel;
    private JLabel statusLabel;
    private JEditorPane viewNodeEditorPane;
    private JTextArea editNodeTextArea;
    private JLabel currentNodeLabel;
    private JTextField getNodeNameTextField;
    private JLabel editNodeLabel;
    private JPanel contentPanel;
    private JPanel mainPanel;
    private JButton editNodeButton;
    private JButton refreshNodeButton;
    private JComboBox nodeRevisionComboBox;
    private JButton backButton;
    private JButton forwardButton;
    private JButton stopButton;

    private class GetNodeButtonActionListener implements ActionListener {

	public void actionPerformed(ActionEvent event) {
	    if (getNodeNameTextField.getText().trim().equals("")) {
		return;
	    }
	    getNode(currentWikiName,
		    getNodeNameTextField.getText().trim(), false);
	}
    }

    private class EditNodeButtonActionListener implements ActionListener {

	public void actionPerformed(ActionEvent event) {
	    cancelGetNode();
	    CardLayout cl = (CardLayout) contentPanel.getLayout();
	    cl.show(contentPanel, "edit");
	}
    }

    private void cancelGetNode() {
	stopButton.setEnabled(false);
	statusLabel.setText("Idle");
	waitingForWikiName = null;
	waitingForNodeName = null;
    }

    private void getNode(final String wikiName, final String nodeName,
			 final boolean refresh) {
	statusLabel.setText("Getting node: " + nodeName + "(" + wikiName + ")");
	stopButton.setEnabled(true);
	waitingForWikiName = wikiName;
	waitingForNodeName = nodeName;
	new Thread(new Runnable() {public void run() {
	    try {
		if (refresh) {
		    wikiManager.refreshNode(wikiName, nodeName);
		}
		final WikiNodeData nodeData = wikiManager.getNode(wikiName, nodeName);
		SwingUtilities.invokeLater(new Runnable() {public void run() {
		    if (waitingForNodeName != null &&
   			    nodeName.equals(waitingForNodeName) &&
			    wikiName.equals(waitingForWikiName)) {
			if (refresh) {
			    historyIndex--;
			}
			view(nodeData);
		    }
		}});
	    } catch (final WikiStoreException ex) {
		SwingUtilities.invokeLater(new Runnable() {public void run() {
		    if (waitingForNodeName != null &&
   			    nodeName.equals(waitingForNodeName) &&
			    wikiName.equals(waitingForWikiName)) {
			cancelGetNode();
		        JOptionPane.showMessageDialog(frame,
		                "There was a problem getting the node: " +
			        ex.getMessage());
		    }
		}});
	    }
	}}).start();
    }

    private void view(WikiNodeData nodeData) {
        while (history.size() > historyIndex + 1) {
	    history.remove(history.size() - 1);
	}
	history.add(nodeData);
	historyIndex++;
	showCurHistNode();
    }

    private void back() {
	historyIndex--;
	showCurHistNode();
    }

    private void forward() {
	historyIndex++;
	showCurHistNode();
    }

    private void showCurHistNode() {
	cancelGetNode();
	WikiNodeData nodeData = (WikiNodeData) history.get(historyIndex);
	currentWikiName = nodeData.getWikiName();
	currentNodeData = nodeData;
	frame.setTitle("FreeWiki: " + currentWikiName);
	currentNodeLabel.setText("Viewing: " + nodeData.getName());
	nodeRevisionComboBox.removeAllItems();
	ArrayList revs = new ArrayList(nodeData.getRevisions());
	Collections.reverse(revs);
	Iterator iter = revs.iterator();
	while (iter.hasNext()) {
	    WikiNodeData.RevisionData rev = (WikiNodeData.RevisionData) iter.next();
	    nodeRevisionComboBox.addItem(rev);
	}
	editNodeLabel.setText("Editing: " + nodeData.getName());
	editNodeButton.setEnabled(true);
	refreshNodeButton.setEnabled(true);
	editNodeTextArea.setText(nodeData.getContent());
	CardLayout cl = (CardLayout) contentPanel.getLayout();
	cl.show(contentPanel, "view");
	backButton.setEnabled(historyIndex > 0);
	forwardButton.setEnabled(historyIndex < history.size() - 1);
    }

    private class SaveNodeRunnable implements Runnable {
	private String wikiName = currentNodeData.getWikiName();
	private String nodeName = currentNodeData.getName();
	private String newContents = editNodeTextArea.getText();
	private int curRevision = currentNodeData.getRevisionNumber();

	public void run() {
	    try {
		final WikiNodeData newData = wikiManager.editNode(wikiName, nodeName,
			newContents, curRevision);
		SwingUtilities.invokeLater(new Runnable() {public void run() {
		    saveDialog.setVisible(false);
		    statusLabel.setText("Idle");
		    historyIndex--;
		    view(newData);
		}});
	    } catch (CollisionException ex) {
		SwingUtilities.invokeLater(new Runnable() {public void run() {
		    saveDialog.setVisible(false);
		    statusLabel.setText("Idle");
		    JOptionPane.showMessageDialog(frame,
		            "The node you are editing has been changed by " +
			    "someone else.  Refresh the node, then edit it " +
			    "again.",
			    "Collision", JOptionPane.WARNING_MESSAGE);
		}});
	    } catch (final WikiStoreException ex) {
		SwingUtilities.invokeLater(new Runnable() {public void run() {
		    saveDialog.setVisible(false);
		    statusLabel.setText("Idle");
		    JOptionPane.showMessageDialog(frame,
		            "There was a problem storing the node: " +
			    ex.getMessage());
		}});
	    }
	}
    }

    private void saveNode() {
	statusLabel.setText("Saving " + currentNodeData.getName() + "(" +
			    currentNodeData.getWikiName() + ")");
	saveDialog.setLocationRelativeTo(frame);
	saveLabel.setText("Saving " + currentNodeData.getName());
	saveDialog.pack();
	new Thread(new SaveNodeRunnable()).start();
	saveDialog.setVisible(true);
    }

    private void addGetNodePanel(Container container) {
	backButton = new JButton("Back");
	backButton.setEnabled(false);
	backButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    back();
		}});
	forwardButton = new JButton("Forward");
	forwardButton.setEnabled(false);
	forwardButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    forward();
		}});
	stopButton = new JButton("Stop");
	stopButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    cancelGetNode();
		}});
	getNodeNameTextField = new JTextField(20);
	getNodeNameTextField.addActionListener(new GetNodeButtonActionListener());
	JButton getNodeButton = new JButton("Get Node");
	getNodeButton.addActionListener(new GetNodeButtonActionListener());
	JPanel getNodePanel = new JPanel();
	getNodePanel.add(backButton);
	getNodePanel.add(forwardButton);
	getNodePanel.add(stopButton);
	getNodePanel.add(getNodeNameTextField);
	getNodePanel.add(getNodeButton);
	container.add(getNodePanel, BorderLayout.NORTH);
    }

    private void addNoNodePanel(Container container) {
	JPanel noNodePanel = new JPanel();
	container.add(noNodePanel, "no");
    }

    private void addViewNodePanel(Container container) {
	currentNodeLabel = new JLabel("");
	nodeRevisionComboBox = new JComboBox();
	nodeRevisionComboBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    WikiNodeData.RevisionData rev = (WikiNodeData.RevisionData)
		            nodeRevisionComboBox.getSelectedItem();
		    if (rev == null) {
			viewNodeEditorPane.setText("");
		    } else {
			viewNodeEditorPane.setText(rev.getContent());
			try {
			    HTMLDocument doc = (HTMLDocument)
				    viewNodeEditorPane.getDocument();
			    URL base = new URL("freewiki://" +
					       currentNodeData.getWikiName() + "/");
			    doc.setBase(base);
			} catch (MalformedURLException ex) {
			    ex.printStackTrace();
			    throw new RuntimeException(ex.toString());
			}
		    }
		}});
	editNodeButton = new JButton("Edit");
	editNodeButton.setEnabled(false);
	editNodeButton.addActionListener(new EditNodeButtonActionListener());
	refreshNodeButton = new JButton("Refresh");
	refreshNodeButton.setEnabled(false);
	refreshNodeButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    getNode(currentNodeData.getWikiName(), currentNodeData.getName(),
			    true);
		}});
	JPanel viewNodeTopPanel = new JPanel();
	viewNodeTopPanel.add(currentNodeLabel);
	viewNodeTopPanel.add(new JLabel("Revision:"));
	viewNodeTopPanel.add(nodeRevisionComboBox);
	viewNodeTopPanel.add(editNodeButton);
	viewNodeTopPanel.add(refreshNodeButton);
	viewNodeEditorPane = new JEditorPane();
	viewNodeEditorPane.setContentType("text/html; charset=UTF-8");
	viewNodeEditorPane.setEditable(false);
	viewNodeEditorPane.addHyperlinkListener(new HyperlinkListener() {
		public void hyperlinkUpdate(HyperlinkEvent event) {
		    if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
			System.out.println("URL=" + event.getURL());
			if (event.getURL().getProtocol().equals("freewiki")) {
			    getNode(FreeWikiURLConnection.wikiName(event.getURL()),
				    FreeWikiURLConnection.nodeName(event.getURL()),
				    false);
			} else {
			    JOptionPane.showMessageDialog(frame,
			            "Cannot get non-freewiki URL.");
			}
		    }
		}});
	JScrollPane nodeContentsScrollPane = new JScrollPane(viewNodeEditorPane);
	JPanel viewNodePanel = new JPanel();
	viewNodePanel.setLayout(new BorderLayout());
	viewNodePanel.add(viewNodeTopPanel, BorderLayout.NORTH);
	viewNodePanel.add(nodeContentsScrollPane, BorderLayout.CENTER);
	container.add(viewNodePanel, "view");
    }

    private void addEditNodePanel(Container container) {
	editNodeLabel = new JLabel("");
	JButton saveNodeButton = new JButton("Save");
	saveNodeButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    saveNode();
		}});
	JButton cancelEditButton = new JButton("Cancel");
	cancelEditButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    view(currentNodeData);
		}});
	JPanel editNodeTopPanel = new JPanel();
	editNodeTopPanel.add(editNodeLabel);
	editNodeTopPanel.add(saveNodeButton);
	editNodeTopPanel.add(cancelEditButton);
	editNodeTextArea = new JTextArea(20, 50);
	JScrollPane nodeContentsScrollPane = new JScrollPane(editNodeTextArea);
	JPanel editNodePanel = new JPanel();
	editNodePanel.setLayout(new BorderLayout());
	editNodePanel.add(editNodeTopPanel, BorderLayout.NORTH);
	editNodePanel.add(nodeContentsScrollPane, BorderLayout.CENTER);
	container.add(editNodePanel, "edit");
    }

    private void addContentPanel(Container container) {
	contentPanel = new JPanel();
	contentPanel.setLayout(new CardLayout());
	addNoNodePanel(contentPanel);
	addViewNodePanel(contentPanel);
	addEditNodePanel(contentPanel);
	container.add(contentPanel, BorderLayout.CENTER);
    }

    private void addStatusLabel(Container container) {
	statusLabel = new JLabel("Idle");
	container.add(statusLabel, BorderLayout.SOUTH);
    }

    private void addComponents(Container contentPane) {
	addGetNodePanel(contentPane);
	addContentPanel(contentPane);
	addStatusLabel(contentPane);
    }

    private void makeSaveDialog() {
	saveDialog = new JDialog(frame, "Saving", true);
	saveDialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	saveLabel = new JLabel("");
	saveDialog.getContentPane().add(saveLabel);
	saveDialog.pack();
    }

    public FreeWikiApplication(WikiManager wikiManager, String wikiName,
			       JFrame frame) {
	this.wikiManager = new CachedWikiManager(wikiManager);
	this.currentWikiName = wikiName;
	this.frame = frame;
	makeSaveDialog();
	addComponents(frame.getContentPane());
	getNode(wikiName, "Root", false);
    }

    public static void main(String[] args) throws IOException {
	if (args.length != 1) {
	    System.out.println("usage: java org.heatdeath.freewiki.gui." +
			       "FreeWikiApplication <wiki-name>");
	    System.exit(0);
	}
	System.out.print("FreeWiki 20010810\n" +
			 "\n" +
			 "ANONYMITY WARNING!  Your anonymity is at risk when using this program\n" +
			 "See the README.txt file for details.\n" +
			 "\n" +
			 "Type YES to continue running the program: ");
	String l = new BufferedReader(new InputStreamReader(System.in)).readLine();
	if (!l.toLowerCase().equals("yes")) {
	    System.out.println("\n\nExiting.\n");
	    System.exit(0);
	}
	System.out.println("\n\nContinuing.\n");
	JFrame frame = new JFrame("FreeWiki: " + args[0]);
	FreeWikiApplication app = new FreeWikiApplication(
		new FreenetWikiManager(System.getProperty("freenet.serverAddress"),
				Integer.parseInt(System.getProperty("freewiki.htl"))),
		args[0], frame);
	frame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    System.exit(0);
		}});
	frame.pack();
	frame.setVisible(true);
    }
}
